;; @(#) tinyfold.el -- extra functions for folding.el

;; @(#) $Id: tinyfold.el,v 1.9 1997/01/15 11:53:13 jaalto NotReleased $
;; @(#) $Keywords: folding, outline, arranging text to blocks $
;;
;; $KnownCompatibility: 18.57 ,19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1994,95,06,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Nov 1 1994
;;
;; To get information on this program use ident(1) or do M-x tifo-version


;; LCD Archive Entry:
;; tinyfold|Jari Aalto|jari.aalto@poboxes.com|
;; tinyfold.el -- extra functions for folding.el|
;; 15-Jan-1997|$Revision: 1.9 $|~/misc/tinyfold.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. The Autoload is preferred.
;;
;;	;;  Normal load
;;	(require 'tinyfold)
;;
;; And optionally
;;
;;      (define-key folding-mode-map "\C-d" 'fold-del)
;;      (define-key folding-mode-map "\C-k" 'fold-kill)
;;
;; IMPORTANT NOTE !!!
;; ========================================
;; - I do not guarrantee, that functions works with multilevel folds!
;;   I use only level 1 fold, no subfolds, since it's easier to open one
;;   fold than go through multilevel dowwwnn...uuuppp, You get the idea.
;;

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVERD prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tifo-" in front of
;;   function & variable. It stands for '(ti)ny (fo)ld
;; - Because this file contains additions to existing module I'm using
;;   fold- as a function prefix for those additional functions.

;;  Preface, 1994
;;
;;	When I used 18.57, I came across with the marvellous packet folding.el
;;	which I immediately started using in daily basis. I knew there existed
;;	outline-mode, but I haven't got the time to dwell into it. It seemed
;;	to depend on STRING that would tell the level of heading, somehow
;;	different approach that folding.el which used rigid FOLD marks {{{ }}}
;;	to restrict the fold region. My choice was folding.el because
;;	it came out of the box and I needed no setup whatsoever.
;;
;;	First I thought that I'll just add these functions to folding.el, but
;;	the Author of it weren't reachable anymore via email. Account did
;;	exist, but no person ever replied to my queries. After I gave a second
;;	thought, it seemed more appropriate to make separate 'plug in'
;;	packet, because these funcs are kinda top level functions.
;;
;;	These are only experimentary functions, I'll try to fix them if
;;	something serious happens.
;;
;;	[Later note: 1997, I may integrate all these funbction to folding.el
;;	but I don't promise anything sure. The development of this package
;;	is freezed]
;;
;;
;;  Overview of features
;;
;;	o   enchancement functions for use with folding.el
;;	o   move to next/prev fold , the pick-move is great !
;;	o   open/close folds in region,delete current fold marks in _this_ fold
;;	o   convert fold marks according to current mode
;;	o   folds/unfolds bunch of buffers matching re
;;
;; Warning
;;
;;	These are not tested under multilevel folds, they may corrupt your
;;	data. Restrict folds to one level only to be sure.

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jan  15      1997	[jari]          19.28	v1.9		NotReleased
;; - Text changes
;;
;; Aug  30      1996	[jari]          19.28	v1.8		NotReleased
;; - XE byteCompiler errors cleared. No code changed, should do that
;;   when I have free time.
;;
;; Feb  10      1996	[jari]          19.28	v1.7		NotReleased
;; - lisp-mnt.el lm-verify checked. Rewrote feedback funcs. ByteComp ok.
;; - **THIS** version is not fully compatible with newest folding.el
;;   that I and anders Lindgren have been developing.
;;
;; Sep  17      1995	[jari]          19.28	v1.6b           NotReleased
;; - We have put a group together to make some enchancements to existing
;;   folding.el. Some of the function were transferred from here to there.
;; - Deleted autoload hack, now there is clean tinyfol-load-hook.
;;
;; Mar  30	1995	[jari]          19.28	v1.6b           NotReleased
;; - Major release.
;; - This a..b versions are the same. Due to time gap I record my
;;   chnages manually, whili it's not different rcs version.
;; + Added function fold-buffers, which is very handy if you're doing
;;   multibuffer replace --> Be sure the buffers are open!
;; + Modified the fold-del a bit, now user can sit on the END marker
;;   to get the fold deleted.
;; + removed zap-to-char, because the implementation is different between
;;   emacs versions. Used my own tifo-ztc instead.
;; + Removed the 'fset' commands because their effect is global,
;;   used funcall instead.
;; + Corrected fold-region-[open|close]. If close was used for
;;   already CLOSED fold, it deleted it..
;;
;; Feb  27	1995	[jari]          19.28	v1.6a           NotReleased
;; - Corrected error in pick-move. LCD entry corr. Now autoload support.
;; + tifo-bool-p ,fold-buffer-p ,fold-re-search-buffer, fold-buffers
;;
;; Dec 	21   	1994  	[jari]          18.57	v1.5            Release_2
;; - added pick-move, really good movement
;; - added install-keys
;;
;; Nov  1-5 	1994  	[jari]          18.57	v1.3            Release_1
;; - First implemented

;;}}}




;;; Code:

;;{{{ setup: variables

;;; ......................................................... &require ...
(require 'folding)

(eval-and-compile
  (autoload 'ti::m-pkg-version-info    "tinylib")
  (autoload 'ti::m-pkg-submit-feedback "tinylib")
  )


;;; ..... Keybingings ..........................................&bind ...

;;; ......................................................... &v-hooks ...

(defvar tinyfold-load-hook nil
  "Hook that is run when package has been loaded.")

;;; ..... user configurable ................................... &conf ...

;;; ..... private variables ................................ &private ...

(defvar tifo-RET nil
  "Global return value from funcs.")

;;}}}

;;{{{ version

;;; ..... version notice ................................... &version ...



(defconst tifo-version-id
  "$Id: tinyfold.el,v 1.9 1997/01/15 11:53:13 jaalto NotReleased $"
  "Latest modification time and version number.")


;;; ----------------------------------------------------------------------
(defun tifo-version ()
  "version information."
  (interactive)
  (require 'tinylib)
  (ti::m-pkg-version-info "tinyfold.el"))

;;; ----------------------------------------------------------------------
;;;
(defun tifo-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (require 'tinylib)
  (ti::m-pkg-submit-feedback "tinyfold.el"))

;;}}}

;;; ########################################################## &macros ###

(defmacro tifo-bool-p (var)
  "Tests if variable is nil or t"
  (list 'or (list 'eq var 'nil) (list 'eq var 't)))

;;; ########################################################### &funcs ###

(defun tifo-ztc(ch)
  "My zap to char implementation.
I started this .el with 18.57, but I found out that they changed the
function definition of zap-to-char in 19.xx, where ztc _deletes_ the last
character in 19, but in 18 it didn't.

This function only zaps to next occurence fwd, not deleting ch.

RETURN: t,nil         ,if killed something
"
  (let ((p (point))
        c ret
        )
    (save-excursion
      (setq c (char-to-string ch))
      (setq ret (search-forward c nil t))
      (if ret
          (kill-region p (1- (point))))
      ret)))


;;; ----------------------------------------------------------------------
(defun tifo-confirm (msg)
  "Confirms action. RET/SPC = ok.
Set global
   TIFO-RET      ,the real character pressed
Return
   nil, t
"
  (let* (ch)
    (message msg) (setq ch (read-char)) (message "")
    (setq tifo-RET ch)
    (cond
     ((or (eq ch ?\015) (eq ch ?\040))  ; RET/SPC
      t)
     (t
      nil))))


;;; ----------------------------------------------------------------------
;;;
(defun fold-buffer-p ()
  "Test if text in buffer is in folded form."
  (let* (;; I think the Beg mark search is enough
	 (bm (nth 0 (fold-get-mode-marks major-mode)))
	 ret
	 )
    (if (null bm) nil			;no mark for this mode
      (setq ret (fold-re-search-buffer (concat "^" bm))))
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun fold-re-search-buffer (re)
  "Test if buffers contains RE. Returns point.
Widens the buffer and narrows back if necessary when done.
Preserves point.
"
  (let* (ret
	 (beg-area (point-min-marker))	;current area begin point
	 (end-area (point-max-marker))	;current max
	 end-wmax			;after widen
	 )
    (unwind-protect
	(progn
	  (widen)
	  (setq end-wmax (point-max))
	  (save-excursion
	    (goto-char (point-min))
	    (if (re-search-forward re nil t)
		(setq ret (point))))
	  )
      ;; what about after widen ? Were we in narrow mode ?
      (if (not (= end-wmax end-area))
	  (narrow-to-region beg-area end-area))
      )
    ret))


;;; ----------------------------------------------------------------------
;;;
(defun fold-buffers (mode re &optional confirm)
  "Folds or unfolds bunch of buffers. Parameters below.
This is usefull if you are going to perform multi buffer replace
eg. with mqr-replace.el.

MODE
 'u' or nil	,unfold
 'f' or t	,fold
RE
 any emacs regexp to find buffers for folding
CONFIRM
 t		,yes
 nil		,no

Return
  count         ,how many buffers touched
"
  (interactive "s[u]nfold/[f]old: \nsBuffer re: \nsConfirm n/[y]:")
  (let* ((blist (buffer-list))
	 (i	0)
	 (ok	t)
	 fn
	 folded			;variables are cheap...

	 msg
	 )

    (catch 'done
      ;;  I know this looks hairy, but accepting CHAR and t/nil
      ;;  causes extra parsing.
      (setq mode
	    (cond
	     ((stringp mode)
	      (cond
	       ((string-match "^[uU]" mode) nil)
	       ((string-match "^[fF]" mode) t)
	       (t "x")))		;not recognized
	     ((or (eq nil mode) (eq t mode))
	      mode)
	     (t "x")))
      (setq confirm
	    (cond
	     ((stringp confirm)
	      (cond
	       ((string-match "^[nN]" confirm) nil)
	       ((string-match "^[yY]" confirm) t)
	       (t t)))			;default is to confirm
	     ((or (eq nil confirm mode) (eq t confirm))
	      confirm)
	     ))

      (if (and (tifo-bool-p mode) (tifo-bool-p confirm)) nil
	(if (interactive-p) (message "Don't know what you want."))
	(throw 'done t))

      (mapcar
       '(lambda (buf)
	  (setq fn (buffer-file-name buf))
	  (if (or (null fn) (null (string-match re fn))) nil
	    (set-buffer buf)
	    (setq folded (fold-buffer-p)) ; has buffer folded text ?

	    (setq msg
		  (concat "proceed fold action [y]/n: "
			  (buffer-name buf)))

	    (if confirm (setq ok (tifo-confirm msg)))

	    (if (null ok)
		(progn (message "..skipped") (sit-for 1))
	      (if (null folded) nil	; no need to look at this
		(folding-mode 1)		; make sure the mode is on
		(cond
		 ((eq nil mode)
		  (fold-open-buffer) (setq i (1+ i)))
		 ((eq t mode)
		  (fold-whole-buffer) (setq i (1+ i)))
		 )))
	    ))
       blist
       ))
    i
    ))








;;; ----------------------------------------------------------------------
;;;
(defun fold-cnv-to-major-folds ()
  "
Many times I use several modes in buffer. Eg. Lisp won't
let me do TAB for comments, so I switch to fundamental-mode.
But I might forget to get back to Lisp-mode before I execute
C-cC-f fold-region --> gets wrong kind of fold marks.

This function replaces smartly all fold marks }}} and {{{
to major mode's fold marks. See folding.el for more info about
fold marks in various modes.

As a side effecs also corrects all foldings to standard notation.
Eg. following ,where correct fold-beg should be \"# {{{ \"
Note that /// marks foldings.


  ///                  ;wrong fold
  #     ///           ;too many spaces, fold format error
  # ///title            ;ok, but title too close

  produces

  #///
  #///
  #/// title

<<< Remember to 'unfold' whole buffer before using this >>>

"
  (interactive)
  (let ((bm "{{{")			; begin match mark
	(em "}}}")			;
	el				; element
	b				; begin
	e				; end
	pp				;
	)

    (catch 'out				; is folding active/loaded ??

      (cond
       ((not (boundp 'fold-mode-marks-alist))
	(message "folding.el not loaded. Cannot convert")
	(throw 'out t)
	))

      (setq el (fold-get-mode-marks major-mode))
      (if (eq nil (nth 0 el)) (throw 'out t))	; ** no mode found

      ;; ok , we're in busines. Search whole buffer and replace.
      (setq b (elt el 0)) (setq e (elt el 1))
      (save-excursion
	;; handle begin mark
	(goto-char (point-min))		; start from beginnig of buf
	(while (re-search-forward bm nil t)
	  ;; set the end position for fold marker
	  (setq pp (point))      (beginning-of-line)
	  (if (looking-at b) 		; should be mode-marked; ok, ignore
	      (goto-char pp)		; note that beg-of-l cmd, move rexp
	    (delete-region (point) pp)
	    (insert b))			; replace with right fold mark
	  )

	;; handle end marks , identical func compared to prev.
	(goto-char (point-min))
	(while (re-search-forward em nil t)
	  (setq pp (point))	  (beginning-of-line)
	  (if (looking-at e)
	      (goto-char pp)
	    (delete-region (point) pp)
	    (insert e)))
	) ;; excur

      ;; ---------------------------------------- catch 'out
      )))




;;; ----------------------------------------------------------------------
;;; - I don't like this function body at all! but I'm lazy to change it.
;;;
(defun fold-del ()
  "
Deletes folding marks that were added with C-c C-f. Usually the
fold to be removed is contextually decided. Normally you're asked
permission to remove the fold first. Normal undo will restore deletion.

Inside sublevel folds:
- If you are inside fold level, and NO subfolds are visible, then the
  current wrapper fold is removed.
- If you are inside fold level, and there IS subfold, then it depends
  on your position what will happen. You'll get information text on
  tricky situations.

Outside all folds:
- if you're inside fold...see prev.
- If you have open fold OR subfold visible when looking forward
  the fold will be opened/removed.


Notes:
   Note that I use /// to mark foldings, 'cause I don't wan't it to become
   confused due to these examples. The [x] marks the current cursor
   position.

Eg 1. If we are inside C-c >, the wrapper fold will  be removed


    /[x]// levelfold    <---  this wrap will be gone
	some text
    \\\                 <--   and this too...and fold level decremented


    /// levelfold
    [x]                 <--  The cursor here, you'll be asked what to do.
    /// subfold ...

    \\\

Eg 2. If we are outside of all folds, then the next fold will be opened
      and the text inside it will become visible.

    [x]
    /// test1 ...     <-- this will be opened and folds removed
    /// test2 ...


    /// test1
    [x]                   ; cursor might also be inside opened fold
    /// opened fold   <-- removed
      text inside
    \\\               <-- this too


"
  (interactive)
  (let* (
	 ch pp p
	 (stack  (length fold-stack))	;How deep we are?
	 (RET ?\015)
	 (SPC ?\040)
	 (MS  "\r")			; string
	 (show-len  40)			; how many chars to dis. 'fold title
	 bm em
	 bmr emr			; beg/end mark rexp

	 in-f				; inside fold flag
	 bfm-p	bbfm-p			; 'beg fold' mark beg.
	 efm-p
	 e				; end
	 del-w				; wrapper
	 del-sf				; subfold
	 el
	 line
	 )

    (catch 'out
      ;;  get mode markers
      (setq el (fold-get-mode-marks major-mode))
      (if (eq nil (nth 0 el)) (throw 'out t))	; ** no mode found

      (setq bm (elt el 0)) (setq em (elt el 1)) ; markers defined for mode
      (setq bmr (concat "^" bm))
      (setq emr (concat "^" em))


      ;;  Are we sitting on ENDING fold mark ?
      ;;  correct the position if so..
      (if (save-excursion
	    (beginning-of-line) (looking-at em))
	  (fold-move-up))

      ;;   ok , we're in busines.
      (save-excursion
	(goto-char (point-max))
	(delete-blank-lines)
	(forward-line -1)		; just adjusting position automatically
	(setq e (point))
	)

      ;; if we are inside fold, the narrow is in effect and point-min <> 1
      (setq in-f (if (eq 1 (point-min)) nil t))

      (beginning-of-line)
      (setq p (point))			; record user's position
      (setq pp (count-lines 1 p))	; this is more accurate position

      ;;   try to locate first BEG fold mark. If we're inside fold there might
      ;;   not be subfolds
      (save-excursion
	(if (eq (point) (point-min))	; must not be at the beg of = inside
	    (forward-line))		; because gives false subfold
	(if (eq nil (re-search-forward bmr nil t)) t ;skip, not found
	  (beginning-of-line) (setq bfm-p (point))))

      ;;   try to locate BEG fold mark upward
      (save-excursion
	(if (eq nil (re-search-backward bmr nil t)) t ;skip, not found
	  (beginning-of-line) (setq bbfm-p (point))))

      ;;
      ;;   try to locate first END fold mark.
      ;;   The text might be C-C C-s opened and cursor inside it
      (save-excursion
	(if (eq (point) (point-max))	;
	    (end-of-line))		;
	(if (eq nil (re-search-forward emr nil t)) t ;skip, not found
	  (beginning-of-line) (setq efm-p (point))))

      (if (eq nil (or efm-p bfm-p bbfm-p)) (throw 'out t)) ;nothing to do

      ;; (d! "FLAGS: in BB b E p pp s" in-f bbfm-p bfm-p efm-p p pp stack)

      (if in-f nil			; not inside  --> subfold then
	(if (> stack 0)			; look at stack
	    (setq del-sf t))
	)

      (if (null in-f) nil
	(if (eq nil bfm-p)		; inside ok but no subfolds
	    (setq del-w t)
	  ;;   inside AND subfolds found
	  ;;   what the user wants to remove ?

	  (message "remove wrapper[cr]/next[spc] fold:")
	  (setq ch (read-char))

	  (if (or (eq ch RET) (eq ch SPC)) t    ; skip,
	    (throw 'out t))		;cancelled operation
	  (if (eq ch SPC)
	      (setq del-sf t)
	    (setq del-w t))
	  ))

      ;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      (if (eq nil del-w) nil
	(fold-exit)			; handle wrap request
	(fold-show)
	(save-excursion
	  (zap-to-char 1 ?\n) (delete-char 1)

	  (goto-char (point-max))
	  (if (eq nil (re-search-backward emr nil t)) nil
	    (tifo-ztc ?\n))
	  )
	(throw 'out t)
	)

      ;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ;; Now there was subfold remove request
      ;; Is the fold opened C-c C-s and user inside it?
      ;; (d! "SUB fold ? " del-sf)

      (if (and del-sf (eq nil efm-p))
	  (if bfm-p			; No E-marker found, so check B-marker
	      (goto-char bfm-p)		; subfold visible
	    (throw 'out t))		; no folds...
	;; E-marker found
	(cond
	 ((looking-at bm) nil)		; user sits on the B-marker
	 ((eq nil bfm-p)
	  (goto-char bbfm-p))		; no subfolds inside {{ }}
	 ((< bfm-p efm-p)		; {{ x {sub }}  {new / there is sub
	  (goto-char bfm-p))
	 ((> bfm-p efm-p)		; {{ x }} {new  / no sub inside
	  (goto-char bbfm-p)		; only inside fold, goto beg mark
	  )
	 (t				; What next ? don't know...
	  nil))
	)



      ;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ;;   fold is opened, so where are we in it?
      ;; (d!  "SUB X " (point))

      (beginning-of-line)
      (save-excursion
	(setq line
	      (buffer-substring
	       (progn (beginning-of-line) (point))
	       (progn (end-of-line) (point))))
	;; Unfortunately there is those nasty ^M chars
	;;
	(if (string-match MS line)
	    (setq line (substring line 0 (match-beginning 0))))
	(if (< (length line) show-len ) nil	; not too long
	  (setq line (substring line 0 show-len))) ;make it shorter
	)

      ;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ;; comfirm the deletion
      (message (concat "del fold [ok]   " line))
      (setq ch (read-char))
      (if (not (eq ch RET)) (throw 'out t)) ; calcelled

      ;;   sitting on "fold closed" ?
      ;;   RE = fold-mark + skip-all + till-hide-mark
      (if (eq nil (looking-at (concat bmr "[^\r]*\r"))) nil ;skip, no...
        ;; (d! "show")
	(fold-show))			; open it !
      (tifo-ztc ?\n)			; ZAP the markers
      (re-search-forward emr nil t)
      (beginning-of-line) (zap-to-char 1 ?\n)

      ;; this is not very smart, but ...
      (goto-line pp)			; restore line position

      ;; ---------------------------------------- catch 'out
      )))






;;; ----------------------------------------------------------------------
;;;
(defun fold-bm-state ()
  "If the cursor sits on the begin mark of fold, this function will
determine if the mark has '...' which means the fold is closed.

Returns:
   0    ,fold is closed
   1    ,fold is open
   nil  ,not a fold mark
"
  (let* ((el (fold-get-mode-marks major-mode))
	 re
	 )
    (if (null el)			;no marks for the mode
	nil
      (save-excursion
	(beginning-of-line)
	(setq re (concat (nth 0 el) ".*" (char-to-string ?\r)))
	(if (looking-at re)
	    0 1)
	))))




;;; ----------------------------------------------------------------------
;;;
(defun fold-region-oc (beg end &optional close)
  "Open/closes all folds inside region. Closes if optional arg is non-nil.
"
  (let* (tmp
	 (ff (if (null close) 'fold-show 'fold-hide))
	 )
    (save-excursion
      ;;   make sure the beg is first.
      (if (> beg end )			;not ok
	  (setq  tmp beg  beg end   end tmp))
      (goto-char  beg)


;;;      (d! "region-oc " (point) beg end)
      (while (and  (fold-move)
		   (< (point) end))
	;;   the fold-show/hide will move point to beg-of-line
	;;   So we must move to the end of line to continue search.
	(if (and close			;already closed
		 (eq 0 (fold-bm-state)))
	    nil
	  (funcall ff) (end-of-line))
	))))




;;; ----------------------------------------------------------------------
;;;
(defun fold-region-open (beg end)
  "opens all folds inside region"
  (interactive "r")
  (fold-region-oc beg end))

(defun fold-region-close (beg end)
  "opens all folds inside region"
  (interactive "r")
  (fold-region-oc beg end 1))



;;; ----------------------------------------------------------------------
;;;
(defun fold-kill ()
  "Kills current fold where user is in. This is for moving folds around
the file.

////   .. if user is within any of these dots, the fold and
       .. the contents are killed
       ..
       ..
       ..
\\\\   ..

"
  (let* ((el (fold-get-mode-marks major-mode))
	 (bm (nth 0 el))
	 (em (nth 1 el))
	 (bmr (concat "^" bm))
	 )
    ;; ****** NOT READY *****
    (catch 'out
      (if (and bm em) nil		;skip, marks ok
	(message "No fold MARKS defined in alist.")
	(throw 'out t))
      (throw 'out t)
      (save-excursion
	(beginning-of-line)
	(if (or (looking-at bmr)	;position at the beginning
		(re-search-forward bmr nil t))
	    ))
      )))




(provide   'tinyfold)
(run-hooks 'tinyfold-load-hook)

;;; tinyfold.el ends here
